#!/usr/bin/python3
# -*- coding: UTF-8 -*-

import sys
import subprocess
from gi.repository import Gtk
from gi.repository import Gdk
import os
from os.path import join, getsize
import dbus
from dbus.mainloop.glib import DBusGMainLoop, threads_init
import gettext
import shutil
import filecmp
import re
import threading
import time
from gi.repository import Gio, GLib, UDisks, GObject


LOCALEDIR = "/usr/share/cga-usb-creator/"
PATH = "/usr/share/cga-usb-creator/"
gettext.bindtextdomain('cga-usb-creator', LOCALEDIR)
gettext.textdomain('cga-usb-creator')

not_interesting = (
	'/org/freedesktop/UDisks2/block_devices/dm',
	'/org/freedesktop/UDisks2/block_devices/ram',
	'/org/freedesktop/UDisks2/block_devices/zram',
	'/org/freedesktop/UDisks2/drives/',
	)

loop_prefix = '/org/freedesktop/UDisks2/block_devices/loop'

GObject.threads_init()
Gdk.threads_init()

def format_size(size):
        """Format a partition size."""
        # Taken from ubiquity's ubiquity/misc.py
        if size == "":
                size = 0
        if size < 1024:
                unit = 'B'
                factor = 1
        elif size < 1024 * 1024:
                unit = 'kB'
                factor = 1024
        elif size < 1024 * 1024 * 1024:
                unit = 'MB'
                factor = 1024 * 1024
        elif size < 1024 * 1024 * 1024 * 1024:
                unit = 'GB'
                factor = 1024 * 1024 * 1024
        else:
                unit = 'TB'
                factor = 1024 * 1024 * 1024 * 1024
        return '%.1f %s' % (float(size) / factor, unit)

class usbcreator_Backend:
	def __init__(self, frontend):
		self.frontend = frontend
		self.devices = {}
		DBusGMainLoop(set_as_default=True)
		threads_init()
		self.bus = dbus.SystemBus()
		self.udisks = UDisks.Client.new_sync(None)

	def detectar_dispositivos(self):
		self.manager = self.udisks.get_object_manager()
		self.manager.connect('object-added', lambda man, obj: self.incluir_dispositivo(obj))
		self.manager.connect('object-removed', lambda man, obj: self.borrar_dispositivo(obj.get_object_path()))

		for obj in self.manager.get_objects():
			self.incluir_dispositivo(obj)

	def incluir_dispositivo(self, obj):
		path = obj.get_object_path()
		for boring in not_interesting:
			if path.startswith(boring):
				return
		block = obj.get_block()
		if not block:
			return
		drive_name = block.get_cached_property('Drive').get_string()
		if drive_name != '/':
			drive = self.udisks.get_object(drive_name).get_drive()
		else:
			drive = None

		if drive and drive.get_cached_property('Optical').get_boolean():
			return

		part = obj.get_partition()
		is_system = block.get_cached_property('HintSystem').get_boolean()
		is_loop = path.startswith(loop_prefix)
		if not (is_system or is_loop):
			if part:
				return
			else:
				self.incluir_dispositivo_udisk(obj, block, drive)
	
	def incluir_dispositivo_udisk(self, obj, block, drive):
		if drive:
			vendor = drive.get_cached_property('Vendor').get_string()
			model = drive.get_cached_property('Model').get_string()
			size = block.get_cached_property('Size').get_uint64()
		else:
			vendor = ''
			model = ''
			size = block.get_cached_property('Size').get_uint64()
		label = 'ETIQUETA'

		if size <= 0:
			return
		key = block.get_cached_property('Device').get_bytestring().decode('utf-8')
		path = obj.get_object_path()
		self.devices[path] = {
			'vendor' : vendor,
			'model' : model,
			'label' : label,
			'mountpoint' : None,
			'free' : -1,
			'capacity' : size,
			'device' : block.get_cached_property('Device').get_bytestring().decode('utf-8')
		}
		self.frontend.actualizar_dispositivos(self.devices)
	
	def borrar_dispositivo(self, path):
		self.frontend.actualizar_dispositivos(self.devices)
		self.devices.pop(path)

class Percent(GObject.GObject, threading.Thread):

        __gsignals__ = {
                         "percent": ( GObject.SIGNAL_RUN_FIRST, GObject.TYPE_NONE, (GObject.TYPE_STRING,))
                        }

        def __init__(self, stream):
                GObject.GObject.__init__(self)
                threading.Thread.__init__(self)
                self.stream = stream

        def emit(self, *args):
                GObject.idle_add(GObject.GObject.emit,self,*args)


        def run(self):
                while True:
                        self.line = self.stream.readline()
                        if len(self.line) == 0:
                                break
                        if len(self.line) <=4:
                                print(self.line)
                                self.emit("percent", self.line)
                                time.sleep(1)


class usbcreator_Frontend():
	def __init__(self):
		#Declara la interfaz gráfica
		self.filename = "/usr/share/cga-usb-creator/cga-usb-creator.glade"
		self.builder = Gtk.Builder()
		self.builder.add_from_file(self.filename)
		#Diccionario
		dic = {"on_btnusb_clicked" : self.btnusb_clicked,
			"on_btndisc_clicked" : self.btndisc_clicked,
			"on_vprincipal_destroy" : self.destroy}
                #Ventanas
		self.principal = self.builder.get_object("vprincipal")
                #se conecta cada señal con la función manejadora              
		self.principal.set_wmclass("Creador de llavero USB", "Creador de llavero USB")
		self.principal.set_default_icon_name('usb-creator-gtk')
		self.builder.connect_signals(dic)
		self.principal.show()
		self.dispositovos = {}

	def btnusb_clicked(self, widget, data=None):
		self.principal.hide()
		self.usb_creator_init()

	def btndisc_clicked(self, widget, data=None):
		self.principal.hide()
		os.spawnl(os.P_NOWAIT,"/usr/bin/usb-creator-gtk","/usr/bin/usb-creator-gtk")
		sys.exit()

	def destroy(self, widget, data=None):
		Gtk.main_quit()

	def usb_creator_init(self):
		self.filename_iso = 'Ninguna'
		self.window = Gtk.Window()
		self.window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
		self.window.set_position(Gtk.WindowPosition.CENTER)
		self.window.set_border_width(10)
		self.window.set_size_request(500, 350)
		self.window.set_title('Creador de llavero USB Guadalinex Edu')
		self.window.set_wmclass("Creador de llavero USB Guadalinex Edu", "Creador de llavero USB Guadalinex Edu")

		self.box = Gtk.VBox(False, 10)
		self.window.add(self.box)

		self.window.connect("delete_event", self.delete_event)
		self.window.connect("destroy", self.destroy)

                # Lista de dispositivos
                # - Etiqueta de la lista
		self.et_lista = Gtk.Label()
		self.et_lista.set_markup('<b>Dispositivos usb conectados</b>')
		self.et_lista.set_use_markup(True)
		self.box.pack_start(self.et_lista, False, True, 0)
                # - Lista
		self.contenedor_lista = Gtk.ScrolledWindow()
		self.listado = Gtk.ListStore(str, str, str, str)
		self.vista_arbol_lista = Gtk.TreeView()
		self.vista_arbol_lista.set_model(self.listado)
		self.contenido_columna_1 = Gtk.CellRendererText()
		self.columna_1 = Gtk.TreeViewColumn('Dispositivo', self.contenido_columna_1, text=0)
		self.vista_arbol_lista.append_column(self.columna_1)
		self.contenido_columna_3 = Gtk.CellRendererText()
		self.columna_3 = Gtk.TreeViewColumn('Capacidad', self.contenido_columna_3, text=2)
		self.vista_arbol_lista.append_column(self.columna_3)
		self.contenido_columna_4 = Gtk.CellRendererText()
		self.columna_4 = Gtk.TreeViewColumn('Etiqueta', self.contenido_columna_4, text=3)
		self.vista_arbol_lista.append_column(self.columna_4)
		self.seleccion_vista = self.vista_arbol_lista.get_selection()
		self.seleccion_vista.connect("changed", self.actualizar_info_dispositivo)
		self.contenedor_lista.add(self.vista_arbol_lista)
		self.contenedor_lista.show_all()
		self.box.pack_start(self.contenedor_lista, True, True, 0)
                # Fin lista de dispositivos
        
                # Cuadro informativo
                # -Cabecera
		self.hbox_info_cab = Gtk.HBox(False, 5)
		self.et_cab_info = Gtk.Label('Dispositivo seleccionado: ')
		self.et_cab_info_dev = Gtk.Label()
		self.et_cab_info_dev.set_markup('<i>dispositivo no seleccionado</i>')
		self.et_cab_info_dev.set_use_markup(True)
		self.hbox_info_cab.pack_start(self.et_cab_info, False, True, 0)
		self.hbox_info_cab.pack_start(self.et_cab_info_dev, False, True, 0)
		self.et_cab_info.show()
		self.et_cab_info_dev.show()
		self.hbox_info_cab.show()
		self.box.pack_start(self.hbox_info_cab, False, True, 0)
                                
		self.hbox_info_pm = Gtk.HBox(False, 5)
		self.b_seleccion_iso = Gtk.Button('Seleccionar imagen ISO')
		self.b_seleccion_iso.set_tooltip_text('Seleccionar imagen ISO')
		self.b_seleccion_iso.set_size_request(120, 30)
		self.b_seleccion_iso.connect("clicked", self.seleccionar_imagen_iso)
		self.b_seleccion_iso.set_sensitive(False)
		self.b_seleccion_iso.show()
		self.hbox_info_pm.pack_start(self.b_seleccion_iso, True, False, 0)
		self.hbox_info_pm.show()
		self.box.pack_start(self.hbox_info_pm, False, True, 0)
		self.hbox_info_iso = Gtk.HBox(False, 5)
		self.et_imagen_sel_info = Gtk.Label('ISO seleccionada: ')
		self.et_imagen_sel_info_dev = Gtk.Label()
		self.et_imagen_sel_info_dev.set_markup('<i>'+self.filename_iso+'</i>')
		self.et_imagen_sel_info_dev.set_use_markup(True)
		self.hbox_info_iso.pack_start(self.et_imagen_sel_info, False, True, 0)
		self.hbox_info_iso.pack_start(self.et_imagen_sel_info_dev, False, True, 0)
		self.et_imagen_sel_info.show()
		self.et_imagen_sel_info_dev.show()
		self.hbox_info_iso.show()
		self.box.pack_start(self.hbox_info_iso, False, True, 0)
                # Fin de cuadro informativo
       
                # -Boton de crear
		self.hbox_boton_cop = Gtk.HBox(False,5)
		self.b_iniciar_cop = Gtk.Button("Crear")
		self.b_iniciar_cop.set_tooltip_text('Crear')
		self.b_iniciar_cop.set_size_request(90, 30)
		self.b_iniciar_cop.connect("clicked", self.create_aviso)
		self.b_iniciar_cop.set_sensitive(False)
		self.hbox_boton_cop.pack_start(self.b_iniciar_cop, True, False, 0)
		self.b_iniciar_cop.show()
		self.hbox_boton_cop.show()
		self.box.pack_start(self.hbox_boton_cop, False, False, 0)
                

		self.vbox_spinner = Gtk.VBox(False, 5) 
		self.spinner = Gtk.Spinner()
		self.spinner.set_size_request(50, 50)
		self.spinner_complete_image = Gtk.Image()
		self.spinner_complete_image.set_from_icon_name("process-completed-symbolic", 48)
		self.label_spinner = Gtk.Label("Operaciones")
		self.vbox_spinner.pack_start(self.spinner, True, False, 0)
		self.vbox_spinner.pack_start(self.spinner_complete_image, True, False, 0)
		self.vbox_spinner.pack_start(self.label_spinner, True, False, 0)
		self.box.pack_start(self.vbox_spinner, False, False, 0)
		self.vbox_spinner.show()

		self.backend = usbcreator_Backend(self)
		self.backend.detectar_dispositivos()

		self.et_lista.show()
		self.box.show() 
		self.window.show()

	def seleccionar_imagen_iso(self, widget, data=None):
		filechooserdialog = Gtk.FileChooserDialog("Seleccionar archivo", parent=self.window,
							action=Gtk.FileChooserAction.OPEN,
							buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN,Gtk.ResponseType.OK))
		filechooserdialog.set_current_folder(os.path.expanduser("~"))
		filtro = Gtk.FileFilter()
		filtro.set_name(".iso")
		filtro.add_pattern("*.iso")
		filechooserdialog.add_filter(filtro)
		response = filechooserdialog.run()
		if response == Gtk.ResponseType.OK:
			self.filename_iso = filechooserdialog.get_filename()
			filechooserdialog.destroy()
			self.file_selected()
		else:
			filechooserdialog.destroy()

	def create_aviso(self, widget, data=None):
                self.mensaje_aviso("Se dispone a borrar su dispositivo.\n¿Desea continuar?", "warning")

	def file_selected(self):
               #Se múitiplica por 1.05 para no tener problemas con usb de 2 gb
		if self.size_bytes > os.path.getsize(self.filename_iso)*1.05:
			self.et_imagen_sel_info_dev.set_markup('<i>'+self.filename_iso+'</i>')
			self.b_iniciar_cop.set_sensitive(True)
		else:
			self.mensaje_aviso("Es necesario un dispositivo de mayor tamaño.", "error")

	def actualizar_dispositivos(self, devices):
		self.dispositivos = {}
		self.listado.clear()
		for device in devices:
			self.dispositivos[devices[device]['device']] = devices[device]
			self.incluir_dispositivo(devices[device])
	
	def incluir_dispositivo(self, disp):
		texto = disp['vendor'] + " " + disp['model'] + " (" + disp['device'] + ")"
		self.listado.append([texto, disp['device'], format_size(disp['capacity']), disp['label']])
	
	def actualizar_info_dispositivo(self, widget, data = None):
		device = None
		(aux, iter) = self.seleccion_vista.get_selected()
		if iter is not None:
			device = aux.get_value(iter, 1)
		else:
			self.et_cab_info_dev.set_markup('<i>dispositivo no seleccionado</i>')
			self.b_seleccion_iso.set_sensitive(False)
			self.b_iniciar_cop.set_sensitive(False)

		if device is not None:
			disp = self.dispositivos[device]
			self.et_cab_info_dev.set_markup(aux.get_value(iter, 0))
			self.b_seleccion_iso.set_sensitive(True)
			self.dispositivo_seleccionado = aux.get_value(iter, 1)
			self.size = aux.get_value(iter, 2)
			self.device = disp['device']
			self.size_bytes = disp['capacity']
			self.partition = disp['device']+"1"

	def delete_event(self, widget, data = None):
		Gtk.main_quit()
        
	def destroy(self, widget, data = None):
		Gtk.main_quit()

	def mensaje_aviso(self, msg, icono): 
		self.cadena = msg
 
                # Ventana del mensaje y contenedor
		self.warning_window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
		self.warning_window = Gtk.Window()
		self.warning_window.set_position(Gtk.WindowPosition.CENTER)
		self.warning_window.set_title('Creador de llavero USB Guadalinex Edu')
		self.warning_window.set_border_width(10)
		self.warning_window.set_deletable(False)
		self.warning_window.set_modal(True)
		self.warning_window.set_transient_for(self.window)
		self.warning_window.set_resizable(False)
		self.warning_window.connect("delete_event", self.delete_event_warning)
		self.warning_window.connect("destroy", self.destroy_warning)
		self.vbox = Gtk.VBox(False, 30)
		self.panel = Gtk.Fixed()
		self.vbox.pack_start(self.panel, True, True, 0)
		self.warning_window.add(self.vbox)

                # Mensaje adaptado
		self.mensaje = Gtk.Label()
		if msg != None:
			self.mensaje.set_text("%s" % self.cadena)

                # Cabecera de la ventana de aviso
		self.imagen_aviso = Gtk.Image()
		self.et_aviso = Gtk.Label()

		if icono == "warning":
			self.window.set_sensitive(False)
			self.boton_cancelar = Gtk.Button('Cancelar')
			self.boton_cancelar.set_size_request(90,30)
			self.boton_cancelar.connect("clicked", self.delete_event_warning, None)
			self.boton_aceptar = Gtk.Button('Aceptar')
			self.boton_aceptar.connect("clicked", self.run_thread_create_usb, None)
			self.boton_aceptar.set_size_request(90,30)
			self.imagen_aviso.set_from_icon_name("dialog-warning-symbolic", 48)
			self.panel.put(self.imagen_aviso, 170, 0)
			self.panel.put(self.mensaje, 70, 60)
			self.panel.put(self.boton_cancelar, 190, 105)
			self.panel.put(self.boton_aceptar, 290, 105)
			self.boton_cancelar.show()
			self.boton_aceptar.show()
		elif icono == "error":
			self.desactivar_durante_create()
			self.imagen_aviso.set_from_icon_name("dialog-error-symbolic", 48)
			self.boton_cancelar = Gtk.Button('Cancelar')
			self.boton_cancelar.set_size_request(90,30)
			self.boton_cancelar.connect("clicked", self.restart, None)
			self.panel.put(self.imagen_aviso, 170, 0)
			self.panel.put(self.mensaje, 45, 60)
			self.panel.put(self.boton_cancelar, 290, 105)
			self.boton_cancelar.show()
		elif icono == "success":
			self.boton_no = Gtk.Button('No')
			self.boton_no.connect("clicked", Gtk.main_quit)
			self.boton_yes = Gtk.Button('Sí')
			self.boton_yes.connect("clicked", self.restart, None)     
			self.boton_no.set_size_request(90,30)
			self.boton_yes.set_size_request(90,30)
			self.imagen_aviso.set_from_icon_name("process-completed-symbolic", 48)
			self.panel.put(self.imagen_aviso, 170, 0)
			self.panel.put(self.mensaje, 70, 60)
			self.panel.put(self.boton_no, 190, 105)
			self.panel.put(self.boton_yes, 290, 105)
			self.boton_no.show()
			self.boton_yes.show()                        

                # Dibujar la ventana de aviso
		self.warning_window.set_size_request(400, 150)
		self.imagen_aviso.show()
		self.mensaje.show()
		self.panel.show()
		self.vbox.show()
		self.warning_window.show()

	def restart(self, widget, data=None):
		self.warning_window.hide()
		self.label_spinner.hide()
		self.spinner.hide()
		self.activar_despues_create()
		self.window.set_deletable(True)

	def delete_event_warning(self, widget, data=None):
		self.warning_window.hide()
		self.window.set_sensitive(True)

	def destroy_warning(self, widget, data=None):
		self.warning_window.destroy()

	def run_thread_create_usb(self, widget, data=None):
		self.warning_window.hide()
		self.window.set_sensitive(True)
		self.window.set_deletable(False)
		self.desactivar_durante_create()
		thread = threading.Thread(target=self.create_usb)
		time.sleep(2)
		thread.start()
		thread.daemon = True
		self.create_usb()
	
	def desactivar_durante_create(self):
		self.b_iniciar_cop.hide()
		
	def activar_despues_create(self):
		self.b_iniciar_cop.show()
		self.filename_iso = 'Ninguna'
		self.et_imagen_sel_info_dev.set_markup('<i>'+self.filename_iso+'</i>')

	def create_usb(self):
		pincho = self.device
		partition = self.partition
		imagen = self.filename_iso
		self.spinner_complete_image.hide()
		self.spinner.start()
		self.spinner.show()
		self.label_spinner.set_text("Desmontando dispositivo")
		self.label_spinner.show()
		return_code = subprocess.call(["sudo", "/usr/share/cga-usb-creator/desmontar_usb.sh", pincho])
		if return_code != 0:
			self.mensaje_aviso("Error al desmontar el dispositivo.", "error")
			return 1
		self.label_spinner.set_text("Montando imagen .iso")
		return_code = subprocess.call(["sudo", "/usr/share/cga-usb-creator/montar_imagen.sh", imagen])
		if return_code != 0:
			self.mensaje_aviso("Error al montar la imagen.", "error")
			return 1
		self.label_spinner.set_text("Borrando particiones")
		return_code = subprocess.call(["sudo", "/usr/share/cga-usb-creator/delete_partition.sh", pincho, partition])
		if return_code != 0:
			self.mensaje_aviso("Error al borrar particiones.", "error")
			return 1
		self.label_spinner.set_text("Sincronizando")
		return_code = subprocess.call(["sudo", "/usr/share/cga-usb-creator/sync.sh"])
		if return_code != 0:
			self.mensaje_aviso("Error al sincronizar.", "error")
			return 1
		
		return_code = subprocess.Popen(["sudo", "/usr/share/cga-usb-creator/pv.sh"], stdout=subprocess.PIPE)
		reader = Percent(return_code.stdout)
		reader.connect("percent", self.update_percent)
		reader.start()
		return_code.wait()
		reader.join()
		if return_code.returncode != 0:
			self.mensaje_aviso("Error al crear la imagen.", "error")
			return 1

		self.label_spinner.set_text("Finalizando proceso")
		return_code = subprocess.call(["sudo", "/usr/share/cga-usb-creator/finished_imagen.sh", pincho])
		if return_code != 0:
			self.mensaje_aviso("Error al finalizar el proceso.", "error")
			return 1
		self.spinner.stop()
		self.spinner.hide()
		self.spinner_complete_image.show()
		self.label_spinner.set_text("Proceso finalizando")
		self.activar_despues_create()
		self.window.set_deletable(True)

	def update_percent(self, sender, percent):
		valor = percent.strip()
		valor_limpio = valor.split("'")[1].split("\\")[0]
		#self.label_spinner.set_text("Creando imagen %s%%" %valor_limpio)
		GObject.idle_add(self.label_spinner.set_text, "Creando imagen %s%%" %valor_limpio)

def main():
	front = usbcreator_Frontend()

if __name__ =="__main__":
	main()
	Gtk.main()
